<?php

namespace Intergo\SubscriptionClient;

use Intergo\BaseClient\Enums\HttpMethodEnum;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\BaseClient\Responses\ErrorResponse;
use Psr\Http\Message\ResponseInterface;

abstract class AbstractSubscriptionService
{
    private SubscriptionClient $client;

    /** @var string */
    protected $token;

    /**
     * AbstractSubscriptionService constructor.
     * https://intergo.atlassian.net/wiki/spaces/ES/pages/2149187587/Communication+between+microservices
     *
     * @param $baseUrl
     * @param $traceId
     */
    public function __construct($baseUrl,$traceId, $token)
    {
        $this->client = new SubscriptionClient($baseUrl, $traceId);
        $this->token = $token;
    }

    /**
     * @param string $endpoint
     * @param $bodyParams
     * @param array $urlParams
     * @return BaseResponseInterface
     */
    protected function execute($endpoint, $bodyParams, array $urlParams = [])
    {
        $bodyParams = $this->prepareBodyData($bodyParams);
        $result = $this->client->execute($endpoint, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

    private function prepareBodyData(array $data)
    {
        return array_merge(['token' => $this->token], $data);
    }

    /**
     * @param string $action
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, BaseResponseInterface $result);

    /**
     * @return array|object
     */
    public function getAllPlans()
    {
        return $this->execute(__FUNCTION__, []);
    }

    /**
     * @param $ip
     * @return array|object
     */
    public function getPlanById($id)
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @param $attributes
     * @return array|object
     */
    public function createPlan($attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $attributes
     * @return array|object
     */
    public function updatePlan($id, $attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->execute(__FUNCTION__, get_defined_vars(),compact('id'));
    }

    /**
     * @param $ip
     * @return array|object
     */
    public function deletePlan($id)
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @return array|object
     */
    public function getPlanTypes()
    {
        return $this->execute(__FUNCTION__,[]);
    }
}